#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

#region Global Enums

public enum ana3BarTrailInput {Close, Median, Typical, Weighted}

#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// The 3 bar trail indicator calculates long and short stop prices from the last 3 bars.
	/// </summary>
	[Description("The 3 bar trail indicator calculates long and short stop prices from the last 3 bars.")]
	public class ana3BarTrail : Indicator
	{
		#region Variables
			private int					index					= 1;
			private int					priorIndex				= 1;
			private double				multiplier				= 0.9;	
			private ana3BarTrailInput 	selectedInput			= ana3BarTrailInput.Typical;
			private bool				showChannel				= true;
			private bool				showMidband				= true;
			private bool				showPaintBars			= true;
			private bool				candles					= true;
			private bool				reverseIntraBar			= false;
			private int					barOpacity				= 0;
			private int					alpha					= 0;
			private int					displacement			= 0;
			private int 				plot0Width 				= 1;
			private PlotStyle 			plot0Style				= PlotStyle.Line;
			private DashStyle			dash0Style				= DashStyle.Dot;
			private int 				plot1Width 				= 2;
			private PlotStyle 			plot1Style				= PlotStyle.Line;
			private DashStyle 			dash1Style				= DashStyle.Dot;
			private int					channelOpacity			= 0;
			private Color 				upColor					= Color.LimeGreen;
			private Color 				downColor				= Color.Red;
			private Color				trendColor				= Color.Transparent;
			private Color				priorColor				= Color.Transparent;
			private BoolSeries 			upTrend;
			private ATR					avgTrueRange;
		
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(Color.Gray, "Mean"));
			Add(new Plot(Color.Gray, "Upper"));
			Add(new Plot(Color.Gray, "Lower"));

			Overlay				= true;
			PlotsConfigurable 	= false;
			BarsRequired		= Math.Max(3, -Displacement);
			upTrend				= new BoolSeries(this);
		}
		
		protected override void OnStartUp()
		{
			avgTrueRange = ATR(2);
			if (ChartControl != null && ChartControl.ChartStyleType == ChartStyleType.CandleStick)
			{
				candles = true;
				alpha = 25*barOpacity;
			}
			else
				candles = false;
			Plots[0].Pen.Width = plot0Width;
			Plots[0].PlotStyle = plot0Style;
			Plots[0].Pen.DashStyle = dash0Style;
			Plots[1].Pen.Width = plot1Width;
			Plots[1].PlotStyle = plot1Style;
			Plots[1].Pen.DashStyle = dash1Style;
			Plots[2].Pen.Width = plot1Width;
			Plots[2].PlotStyle = plot1Style;
			Plots[2].Pen.DashStyle = dash1Style;
			index = BarsRequired;
		}		
		
		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
			if(CurrentBar < 1)
			{
				Upper.Set(High[0] + TickSize);
				Lower.Set(Low[0] - TickSize);
				Value.Set(0.5 * (Upper[0] + Lower[0]));
				upTrend.Set(true);
				return;
			}
			else if (CurrentBar < BarsRequired) 
			{
				Upper.Set(High[1] + TickSize);
				Lower.Set(Low[1] - TickSize);
				Value.Set(0.5 * (Upper[0] + Lower[0]));
				upTrend.Set(true);
				trendColor = upColor;
				return;
			}

			if(FirstTickOfBar)
			{
				if(selectedInput == ana3BarTrailInput.Close)
				{	
					Upper.Set(Math.Max(High[1] + TickSize, 0.5 * (Close[2] + Close[3]) + multiplier * avgTrueRange[2]));
					Lower.Set(Math.Min(Low[1] - TickSize, 0.5 * (Close[2] + Close[3]) - multiplier * avgTrueRange[2]));
				}	
				else if(selectedInput == ana3BarTrailInput.Median)
				{	
					Upper.Set(Math.Max(High[1] + TickSize, 0.5 * (Median[2] + Median[3]) + multiplier * avgTrueRange[2]));
					Lower.Set(Math.Min(Low[1] - TickSize, 0.5 * (Median[2] + Median[3]) - multiplier * avgTrueRange[2]));
				}	
				else if(selectedInput == ana3BarTrailInput.Typical)
				{	
					Upper.Set(Math.Max(High[1] + TickSize, 0.5 * (Typical[2] + Typical[3]) + multiplier * avgTrueRange[2]));
					Lower.Set(Math.Min(Low[1] - TickSize, 0.5 * (Typical[2] + Typical[3]) - multiplier * avgTrueRange[2]));
				}	
				else if(selectedInput == ana3BarTrailInput.Weighted)
				{	
					Upper.Set(Math.Max(High[1] + TickSize, 0.5 * (Weighted[2] + Weighted[3]) + multiplier * avgTrueRange[2]));
					Lower.Set(Math.Min(Low[1] - TickSize, 0.5 * (Weighted[2] + Weighted[3]) - multiplier * avgTrueRange[2]));
				}	
				Value.Set(0.5 * (Upper[0] + Lower[0]));
				displacement = Math.Max(Displacement, -priorIndex);
				priorIndex = index;
				priorColor = trendColor;
			}	
			
			if(reverseIntraBar)
			{
				if(High[0] > Upper[0] && !(Low[0] < Lower[0] && !upTrend[1]))
				{
					upTrend.Set(true);
					trendColor = upColor;
				}	
				else if(Low[0] < Lower[0] && !(High[0] > Upper[0] && upTrend[1]))
				{
					upTrend.Set(false);
					trendColor = downColor;
				}	
				else
				{
					upTrend.Set(upTrend[1]);
					trendColor = priorColor;
				}	
			}	
			else	
			{	
				if(Close[0] > Upper[0])
				{
					upTrend.Set(true);
					trendColor = upColor;
				}	
				else if(Close[0] < Lower[0])
				{
					upTrend.Set(false);
					trendColor = downColor;
				}	
				else
				{
					upTrend.Set(upTrend[1]);
					trendColor = priorColor;
				}	
			}
			
			if(showMidband)
				PlotColors[0][-displacement] = trendColor;
			else
				PlotColors[0][-displacement] = Color.Transparent;
			
			if(showChannel)
			{
				PlotColors[1][-displacement] = trendColor;
				PlotColors[2][-displacement] = trendColor;
			}
			else
			{
				PlotColors[1][-displacement] = Color.Transparent;
				PlotColors[2][-displacement] = Color.Transparent;
			}	
			
			if (channelOpacity != 0)
			{	
				if(upTrend[0] == upTrend[1])
				{
					if(index == CurrentBar)
						RemoveDrawObject("channel" + CurrentBar);
					DrawRegion("channel" + priorIndex, CurrentBar - priorIndex + 1 - displacement, - displacement, Upper, Lower, Color.Transparent, trendColor, channelOpacity);
					index = priorIndex;
				}
				else
				{
					if (index == priorIndex)
						DrawRegion("channel" + priorIndex, CurrentBar - priorIndex + 1 - displacement, 1 - displacement, Upper, Lower, Color.Transparent, priorColor, channelOpacity);
					DrawRegion("channel" + CurrentBar, 1 - displacement, -displacement, Upper, Lower, Color.Transparent, trendColor, channelOpacity);
					index = CurrentBar;
				}	
			}	
			
			if(showPaintBars)
			{
				if(candles && Close[0] > Open[0])
					BarColorSeries[-displacement] = Color.FromArgb(alpha, trendColor);
				else
					BarColorSeries[-displacement] = trendColor;
				CandleOutlineColorSeries[-displacement] = trendColor;
			}
		}
		
		#region Properties
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Mean
		{
			get { return Values[0]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Upper
		{
			get { return Values[1]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Lower
		{
			get { return Values[2]; }
		}
			
		[Browsable(false)]
        [XmlIgnore()]		
        public BoolSeries UpTrend
        {
            get { return upTrend; }
        }

		[Description("ATR multiplier used to calculate stop bands")]
        [GridCategory("Parameters")]
		[Gui.Design.DisplayName ("ATR multiplier")]
        public double Multiplier
        {
            get { return multiplier; }
            set { multiplier = Math.Max(0, value); }
        }

		/// <summary>
		/// </summary>
		[Description("Formula used to calculate stop bands")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Calculate bands from")]
		public ana3BarTrailInput SelectedInput
		{
			get { return selectedInput; }
			set { selectedInput = value; }
		}
		
		[Description("Reverse intra-bar")]
        [GridCategory("Parameters")]
		[Gui.Design.DisplayName ("Reverse intra-bar")]
        public bool ReverseIntraBar
        {
            get { return reverseIntraBar; }
            set { reverseIntraBar = value; }
        }

		[Description("Show channel lines. ")]
        [Category("Options")]
		[Gui.Design.DisplayName ("Show channel lines")]
        public bool ShowChannel
        {
            get { return showChannel; }
            set { showChannel = value; }
        }

		[Description("Show midband of channel. ")]
        [Category("Options")]
		[Gui.Design.DisplayName ("Show midband")]
        public bool ShowMidband
        {
            get { return showMidband; }
            set { showMidband = value; }
        }

		[Description("Show paint bars.")]
        [Category("Options")]
		[Gui.Design.DisplayName ("Show paintbars")]
        public bool ShowPaintBars
        {
            get { return showPaintBars; }
            set { showPaintBars = value; }
        }

		/// <summary>
		/// </summary>
        [XmlIgnore()]		
		[Description("Select color for uptrend")]
		[Category("Plot Colors")]
		[Gui.Design.DisplayName("Uptrend")]
		public Color UpColor
		{
			get { return upColor; }
			set { upColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string UpColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(upColor); }
			set { upColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
        [XmlIgnore()]		
		[Description("Select color for downtrend")]
		[Category("Plot Colors")]
		[Gui.Design.DisplayName("Downtrend")]
		public Color DownColor
		{
			get { return downColor; }
			set { downColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string DownColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(downColor); }
			set { downColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[Description("Width for midband")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Width midband")]
		public int Plot0Width
		{
			get { return plot0Width; }
			set { plot0Width = Math.Max(1, value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("PlotStyle for midband")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Plot style midband")]
		public PlotStyle Plot0Style
		{
			get { return plot0Style; }
			set { plot0Style = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("DashStyle for midband")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Dash style midband")]
		public DashStyle Dash0Style
		{
			get { return dash0Style; }
			set { dash0Style = value; }
		} 
		
		/// <summary>
		/// </summary>
		[Description("Width for channel lines")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Width channel lines")]
		public int Plot1Width
		{
			get { return plot1Width; }
			set { plot1Width = Math.Max(1, value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("PlotStyle for channel lines")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Plot style channel lines")]
		public PlotStyle Plot1Style
		{
			get { return plot1Style; }
			set { plot1Style = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("DashStyle for channel lines")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Dash style channel lines")]
		public DashStyle Dash1Style
		{
			get { return dash1Style; }
			set { dash1Style = value; }
		} 

		[Description("Select opacity for the area between the upper and lower band of the Donchian Channel.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Channel opacity")]
		public int ChannelOpacity
		{
			get { return channelOpacity; }
			set { channelOpacity = value; }
		}

		[Description("Select opacity for the upclose paintbars.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Paintbar opacity")]
		public int BarOpacity
		{
			get { return barOpacity; }
			set { barOpacity = value; }
		}
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private ana3BarTrail[] cacheana3BarTrail = null;

        private static ana3BarTrail checkana3BarTrail = new ana3BarTrail();

        /// <summary>
        /// The 3 bar trail indicator calculates long and short stop prices from the last 3 bars.
        /// </summary>
        /// <returns></returns>
        public ana3BarTrail ana3BarTrail(double multiplier, bool reverseIntraBar, ana3BarTrailInput selectedInput)
        {
            return ana3BarTrail(Input, multiplier, reverseIntraBar, selectedInput);
        }

        /// <summary>
        /// The 3 bar trail indicator calculates long and short stop prices from the last 3 bars.
        /// </summary>
        /// <returns></returns>
        public ana3BarTrail ana3BarTrail(Data.IDataSeries input, double multiplier, bool reverseIntraBar, ana3BarTrailInput selectedInput)
        {
            if (cacheana3BarTrail != null)
                for (int idx = 0; idx < cacheana3BarTrail.Length; idx++)
                    if (Math.Abs(cacheana3BarTrail[idx].Multiplier - multiplier) <= double.Epsilon && cacheana3BarTrail[idx].ReverseIntraBar == reverseIntraBar && cacheana3BarTrail[idx].SelectedInput == selectedInput && cacheana3BarTrail[idx].EqualsInput(input))
                        return cacheana3BarTrail[idx];

            lock (checkana3BarTrail)
            {
                checkana3BarTrail.Multiplier = multiplier;
                multiplier = checkana3BarTrail.Multiplier;
                checkana3BarTrail.ReverseIntraBar = reverseIntraBar;
                reverseIntraBar = checkana3BarTrail.ReverseIntraBar;
                checkana3BarTrail.SelectedInput = selectedInput;
                selectedInput = checkana3BarTrail.SelectedInput;

                if (cacheana3BarTrail != null)
                    for (int idx = 0; idx < cacheana3BarTrail.Length; idx++)
                        if (Math.Abs(cacheana3BarTrail[idx].Multiplier - multiplier) <= double.Epsilon && cacheana3BarTrail[idx].ReverseIntraBar == reverseIntraBar && cacheana3BarTrail[idx].SelectedInput == selectedInput && cacheana3BarTrail[idx].EqualsInput(input))
                            return cacheana3BarTrail[idx];

                ana3BarTrail indicator = new ana3BarTrail();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Multiplier = multiplier;
                indicator.ReverseIntraBar = reverseIntraBar;
                indicator.SelectedInput = selectedInput;
                Indicators.Add(indicator);
                indicator.SetUp();

                ana3BarTrail[] tmp = new ana3BarTrail[cacheana3BarTrail == null ? 1 : cacheana3BarTrail.Length + 1];
                if (cacheana3BarTrail != null)
                    cacheana3BarTrail.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheana3BarTrail = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The 3 bar trail indicator calculates long and short stop prices from the last 3 bars.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ana3BarTrail ana3BarTrail(double multiplier, bool reverseIntraBar, ana3BarTrailInput selectedInput)
        {
            return _indicator.ana3BarTrail(Input, multiplier, reverseIntraBar, selectedInput);
        }

        /// <summary>
        /// The 3 bar trail indicator calculates long and short stop prices from the last 3 bars.
        /// </summary>
        /// <returns></returns>
        public Indicator.ana3BarTrail ana3BarTrail(Data.IDataSeries input, double multiplier, bool reverseIntraBar, ana3BarTrailInput selectedInput)
        {
            return _indicator.ana3BarTrail(input, multiplier, reverseIntraBar, selectedInput);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The 3 bar trail indicator calculates long and short stop prices from the last 3 bars.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ana3BarTrail ana3BarTrail(double multiplier, bool reverseIntraBar, ana3BarTrailInput selectedInput)
        {
            return _indicator.ana3BarTrail(Input, multiplier, reverseIntraBar, selectedInput);
        }

        /// <summary>
        /// The 3 bar trail indicator calculates long and short stop prices from the last 3 bars.
        /// </summary>
        /// <returns></returns>
        public Indicator.ana3BarTrail ana3BarTrail(Data.IDataSeries input, double multiplier, bool reverseIntraBar, ana3BarTrailInput selectedInput)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.ana3BarTrail(input, multiplier, reverseIntraBar, selectedInput);
        }
    }
}
#endregion
